<?php

namespace Drupal\o365_sso\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\externalauth\ExternalAuth;
use Drupal\o365\AuthenticationService;
use Drupal\o365\GraphService;
use Drupal\o365\O365ConnectorInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * The UserLoginController class.
 */
class UserLoginController extends ControllerBase {

  /**
   * Drupal\o365\GraphService definition.
   *
   * @var \Drupal\o365\GraphService
   */
  protected $graphService;

  /**
   * Drupal\o365\AuthenticationService definition.
   *
   * @var \Drupal\o365\AuthenticationService
   */
  protected $authenticationService;

  /**
   * The external auth service.
   *
   * @var \Drupal\externalauth\ExternalAuth
   */
  protected $externalAuth;

  /**
   * Constructs a new UserLoginController object.
   *
   * @param \Drupal\o365\GraphService $o365_graph
   *   The GraphService definition.
   * @param \Drupal\o365\AuthenticationService $authentication_service
   *   The AuthenticationService definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManager definition.
   * @param \Drupal\externalauth\ExternalAuth $external_auth
   *   The external auth.
   */
  public function __construct(
    GraphService $o365_graph,
    AuthenticationService $authentication_service,
    EntityTypeManagerInterface $entity_type_manager,
    ExternalAuth $external_auth
  ) {
    $this->graphService = $o365_graph;
    $this->authenticationService = $authentication_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->externalAuth = $external_auth;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('o365.graph'),
      $container->get('o365.authentication'),
      $container->get('entity_type.manager'),
      $container->get('externalauth.externalauth')
    );
  }

  /**
   * Login a user.
   *
   * @param \Drupal\o365\O365ConnectorInterface $o365_connector
   *   The Office 365 connector entity.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The redirect to the set URL in config.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   */
  public function login(O365ConnectorInterface $o365_connector) {
    // Enable cookie auth.
    $this->authenticationService->useCookieAuth = TRUE;

    // Get user data.
    $userData = $this->graphService->getGraphData('/me');

    // Get redirect url after login.
    $redirectUrl = $o365_connector->getRedirectLogin();

    // Get user unique identifier.
    $o365_id = $userData['id'];
    $account = $this->externalAuth->load($o365_id, 'o365_sso');

    // Check for email match.
    if (!empty($account)) {
      $email = $userData['userPrincipalName'];
      $account = $this->entityTypeManager->getStorage('user')->loadByProperties(['mail' => $email]);
      $account = !empty($account) ? reset($account) : FALSE;
    }

    if ($account) {
      /** @var \Drupal\user\UserInterface $account */
      $this->externalAuth->linkExistingAccount($o365_id, 'o365_sso', $account);
    }

    // If we haven't found an account to link, create one from the data.
    if (!$account) {
      $accountData = [
        'name' => $userData['userPrincipalName'],
        'mail' => $userData['userPrincipalName'],
      ];
      $account = $this->externalAuth->register($o365_id, 'o365_sso', $accountData);
    }
    elseif ($account->isBlocked()) {
      $this->messenger()->addError(t('The requested account is blocked'));
      return $this->redirect('user.login');
    }

    // Log the user in.
    $this->externalAuth->userLoginFinalize($account, $o365_id, 'o365_sso');

    // Save the auth data in the session and delete the cookie.
    $this->authenticationService->saveAuthDataFromCookie();

    // Return the redirect.
    return new RedirectResponse($redirectUrl);
  }

}
