<?php

namespace Drupal\o365_sso_user\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\externalauth\Event\ExternalAuthEvents;
use Drupal\externalauth\Event\ExternalAuthLoginEvent;
use Drupal\o365\GraphService;

/**
 * The event subscriber for SSO logins.
 *
 * @package Drupal\o365_sso\EventSubscriber
 */
class UserLoginEventSubscriber implements EventSubscriberInterface {

  /**
   * Drupal\Core\Config\ImmutableConfig definition.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Drupal\o365\GraphService definition.
   *
   * @var \Drupal\o365\GraphService
   */
  protected $graphService;

  /**
   * UserLoginEventSubscriber constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\o365\GraphService $o365_graph
   *   The graph service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, GraphService $o365_graph) {
    $this->config = $config_factory->get('o365_sso_user.settings');
    $this->graphService = $o365_graph;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ExternalAuthEvents::LOGIN][] = ['updateUserData'];
    return $events;
  }

  /**
   * Update a users data with data dfrom the Graph API.
   *
   * @param \Drupal\externalauth\Event\ExternalAuthLoginEvent $event
   *   The ExternalAuthLoginEvent event containing the user.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   */
  public function updateUserData(ExternalAuthLoginEvent $event) {
    // Only do this when we want to use graph data.
    if (!empty($this->config->get('use_graph_data'))) {
      $user = $event->getAccount();

      // Get user data.
      $userData = $this->graphService->getGraphData('/me');
      $mapping = $this->getFieldMapping();

      $userAltered = FALSE;
      // Loop through the mappings if needed.
      if (!empty($mapping) && !empty($userData)) {
        foreach ($mapping as $userDataField => $userField) {
          // Check if the fields exist in both the data and the user.
          if (isset($userData[$userDataField]) && $user->hasField($userField)) {
            $user->set($userField, $userData[$userDataField]);
            $userAltered = TRUE;
          }
        }
      }

      // Save the user when it has changed.
      if ($userAltered) {
        $user->save();
      }
    }
  }

  /**
   * Get the field mappings from the settings.
   *
   * @return array
   *   The mapping as graph_api_field => drupal_field.
   */
  protected function getFieldMapping() {
    $mapping = [];
    $data = $this->config->get('graph_data_mapping');
    $lines = preg_split('/\r\n|\r|\n/', $data);

    if (!empty($lines)) {
      foreach ($lines as $line) {
        $field = explode('|', $line);
        $mapping[trim($field[0])] = trim($field[1]);
      }
    }

    return $mapping;
  }
}