<?php

namespace Drupal\o365;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Provider\GenericProvider;
use League\OAuth2\Client\Token\AccessTokenInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * The AuthenticationService class.
 */
class AuthenticationService implements AuthenticationServiceInterface {

  /**
   * The config factory interface.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The modules base config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $moduleConfig;

  /**
   * The private temp store.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStore;

  /**
   * A oauth provider.
   *
   * @var \League\OAuth2\Client\Provider\GenericProvider
   */
  protected $oauthClient;

  /**
   * The ConstantsService implementation.
   *
   * @var \Drupal\o365\ConstantsService
   */
  protected $constants;

  /**
   * The logger service.
   *
   * @var \Drupal\o365\O365LoggerServiceInterface
   */
  protected $loggerService;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * If we want to add debug messages.
   *
   * @var bool
   */
  private $debug;

  /**
   * The auth data.
   *
   * @var array
   */
  private $authValues = [];

  /**
   * If we want to save the auth as a cookie.
   *
   * @var bool
   */
  public $useCookieAuth = FALSE;

  /**
   * Constructs a new AuthenticationService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory interface.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $tempStoreFactory
   *   The private store factory.
   * @param \Drupal\o365\ConstantsService $constantsService
   *   The constants service from the o365 module.
   * @param \Drupal\o365\O365LoggerServiceInterface $loggerService
   *   The logger service from the o365 module.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    PrivateTempStoreFactory $tempStoreFactory,
    ConstantsService $constantsService,
    O365LoggerServiceInterface $loggerService,
    RequestStack $requestStack,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->configFactory = $configFactory;
    $this->moduleConfig = $this->configFactory->get('o365.settings');
    $this->tempStore = $tempStoreFactory;
    $this->constants = $constantsService;
    $this->loggerService = $loggerService;
    $this->request = $requestStack->getCurrentRequest();
    $this->entityTypeManager = $entity_type_manager;

    $this->debug = !empty($this->moduleConfig->get('verbose_logging'));
  }

  /**
   * {@inheritdoc}
   */
  public function loginUser(O365ConnectorInterface $o365_connector) {
    if ($this->debug) {
      $message = t('-- loginUser()');
      $this->loggerService->debug($message);
    }

    $clientId = $o365_connector->getClientId();

    if ($this->debug) {
      $message = t('clientId: @client', ['@client' => $clientId]);
      $this->loggerService->debug($message);
    }

    $this->generateProvider($o365_connector);
    $authUrl = $this->oauthClient->getAuthorizationUrl() . '&client_id=' . $clientId;
    $response = new TrustedRedirectResponse($authUrl);

    return $response->send();
  }

  /**
   * {@inheritdoc}
   */
  public function setAccessToken(string $code, O365ConnectorInterface $o365_connector, $redirect = FALSE) {
    try {
      // Make the token request.
      $this->generateProvider($o365_connector);
      $accessToken = $this->oauthClient->getAccessToken('authorization_code', [
        'code' => $code,
        'client_id' => $o365_connector->getClientId(),
        'client_secret' => $o365_connector->getClientSecret(),
      ]);

      $this->saveAuthData($accessToken, $o365_connector);

      if ($redirect) {
        $response = new TrustedRedirectResponse($redirect);
        return $response->send();
      }
    }
    catch (IdentityProviderException $e) {
      $error = $e->getResponseBody();
      $message = t('Error description: @error', ['@error' => $error['error_description']]);
      $this->loggerService->log($message, 'error');
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getAccessToken(bool $login = FALSE) {
    $authData = $this->getAuthData();

    if (!empty($authData)) {
      $now = time();

      if ($authData['expires_on'] > $now) {
        return $authData['access_token'];
      }

      if ($this->debug) {
        $message = t('accessToken is expired, refresh the token');
        $this->loggerService->debug($message);
      }

      return $this->refreshToken($authData['refresh_token'], $authData['connector_id']);
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function saveAuthDataFromCookie(): void {
    $authCookie = $_COOKIE[$this->constants->getUserTempStoreDataName()];
    $authValues = Json::decode($authCookie);

    // Save the data and delete the cookie.
    $this->saveDataToTempStore($this->constants->getUserTempStoreDataName(), $authValues);
    setcookie($this->constants->getUserTempStoreDataName(), '', time() - 3600);

    $this->useCookieAuth = FALSE;
  }

  /**
   * Generate a new access token based on the refresh token.
   *
   * @param string $refreshToken
   *   The refresh token.
   * @param string $connector_id
   *   The ID of used connector.
   *
   * @return string
   *   The new access token.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   */
  private function refreshToken(string $refreshToken, string $connector_id): string {
    /** @var \Drupal\o365\O365ConnectorInterface $o365_connector */
    $o365_connector = $this->entityTypeManager->getStorage('o365_connector')->load($connector_id);
    $this->generateProvider($o365_connector);

    $accessToken = $this->oauthClient->getAccessToken('refresh_token', [
      'refresh_token' => $refreshToken,
      'client_id' => $o365_connector->getClientId(),
      'client_secret' => $o365_connector->getClientSecret(),
    ]);
    $this->saveAuthData($accessToken, $o365_connector);
    return $accessToken->getToken();
  }

  /**
   * Generate a basic oAuth2 provider.
   *
   * @param \Drupal\o365\O365ConnectorInterface $o365_connector
   *   The Office 365 connector entity.
   */
  private function generateProvider(O365ConnectorInterface $o365_connector): void {
    $scopes = $o365_connector->getAuthScopes();
    $redirect_uri = Url::fromRoute(
      'o365_sso.login_callback_controller_callback',
      ['o365_connector' => $o365_connector->id()],
      ['absolute' => TRUE]
    )->toString();

    $providerData = [
      'client_id' => $o365_connector->getClientId(),
      'client_secret' => $o365_connector->getClientSecret(),
      'redirectUri' => $redirect_uri,
      'urlAuthorize' => $this->constants->getAuthorizeUrl($o365_connector),
      'urlAccessToken' => $this->constants->getTokenUrl($o365_connector),
      'urlResourceOwnerDetails' => '',
      'scopes' => (strpos($scopes, 'offline_access') !== FALSE) ? $scopes : 'offline_access ' . $scopes,
    ];

    if ($this->debug) {
      $message = t('providerData: @data', ['@data' => print_r($providerData, 1)]);
      $this->loggerService->debug($message);
    }

    $this->oauthClient = new GenericProvider($providerData);
  }

  /**
   * Get the auth data from temp store or cookie.
   *
   * @return array
   *   The saved auth data.
   */
  private function getAuthData(): array {
    if ($this->useCookieAuth) {
      $authCookie = $_COOKIE[$this->constants->getUserTempStoreDataName()];
      return Json::decode($authCookie);
    }
    $tempstore = $this->tempStore->get($this->constants->getUserTempStoreName());

    return $tempstore->get($this->constants->getUserTempStoreDataName());
  }

  /**
   * Save the auth data to the temp store.
   *
   * @param \League\OAuth2\Client\Token\AccessTokenInterface $accessToken
   *   The access token object.
   * @param \Drupal\o365\O365ConnectorInterface $o365_connector
   *   The Office 365 connector entity.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  private function saveAuthData(AccessTokenInterface $accessToken, O365ConnectorInterface $o365_connector): void {
    $this->authValues = [
      'access_token' => $accessToken->getToken(),
      'refresh_token' => $accessToken->getRefreshToken(),
      'expires_on' => $accessToken->getExpires(),
      'connector_id' => $o365_connector->id(),
    ];

    if ($this->debug) {
      $message = t('Saving authData: @data', ['@data' => print_r($this->authValues, 1)]);
      $this->loggerService->debug($message);
    }

    $this->saveDataToTempStore($this->constants->getUserTempStoreDataName(), $this->authValues);
    setcookie($this->constants->getUserTempStoreDataName(), Json::encode($this->authValues), $this->constants->getCookieExpire(), '/');
  }

  /**
   * Save data to the tempstore.
   *
   * @param string $name
   *   The name of the store.
   * @param mixed $value
   *   The value.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  private function saveDataToTempStore(string $name, $value): void {
    $tempstore = $this->tempStore->get($this->constants->getUserTempStoreName());
    $tempstore->set($name, $value);
  }

}
