<?php

namespace Drupal\o365;

use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * The ConstantsService class.
 */
class ConstantsService {

  /**
   * The config factory interface.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The url where Microsoft will redirect us too.
   *
   * @var string
   */
  private $redirectUrl = '/o365/callback';

  /**
   * The authorize endpoint root.
   *
   * @var string
   */
  private $authorizeRoot = 'https://login.microsoftonline.com/';

  /**
   * The authorize endpoint path.
   *
   * @var string
   */
  private $authorizePath = '/oauth2/v2.0/authorize';

  /**
   * The token endpoint root.
   *
   * @var string
   */
  private $tokenRoot = 'https://login.microsoftonline.com/';

  /**
   * The token endpoint path.
   *
   * @var string
   */
  private $tokenPath = '/oauth2/v2.0/token';

  /**
   * The name of the temp store.
   *
   * @var string
   */
  private $userTempStoreName = 'o365.tempstore';

  /**
   * The name of the data saved in the temp store.
   *
   * @var string
   */
  private $userTempStoreDataName = 'o365AuthData';

  /**
   * Constructs a new ConstantsService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory interface.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack object.
   */
  public function __construct(ConfigFactoryInterface $configFactory, RequestStack $request_stack) {
    $this->configFactory = $configFactory;
    $this->request = $request_stack->getCurrentRequest();
  }

  /**
   * Get the redirect URL.
   *
   * @return string
   *   The redirect url.
   */
  public function getRedirectUrl(): string {
    return 'https://' . $this->request->getHost() . $this->redirectUrl;
  }

  /**
   * Get the authorize url.
   *
   * @param \Drupal\o365\O365ConnectorInterface $o365_connector
   *   The Office 365 connector entity.
   *
   * @return string
   *   The authorize url.
   */
  public function getAuthorizeUrl(O365ConnectorInterface $o365_connector): string {
    $tenant = empty($o365_connector->getTenantId()) ? 'common' : $o365_connector->getTenantId();

    return $this->authorizeRoot . $tenant . $this->authorizePath;
  }

  /**
   * Get the token url.
   *
   * @param \Drupal\o365\O365ConnectorInterface $o365_connector
   *   The Office 365 connector entity.
   *
   * @return string
   *   The token url.
   */
  public function getTokenUrl(O365ConnectorInterface $o365_connector): string {
    $tenant = empty($o365_connector->getTenantId()) ? 'common' : $o365_connector->getTenantId();

    return $this->tokenRoot . $tenant . $this->tokenPath;
  }

  /**
   * Get the user temp store name.
   *
   * @return string
   *   The user temp store name.
   */
  public function getUserTempStoreName(): string {
    return $this->userTempStoreName;
  }

  /**
   * Get the user temp store data name.
   *
   * @return string
   *   The user temp store data name.
   */
  public function getUserTempStoreDataName(): string {
    return $this->userTempStoreDataName;
  }

  /**
   * Get the cookie expire timestamp.
   *
   * @return int
   *   The expire timestamp.
   */
  public function getCookieExpire(): int {
    return time() + 3600;
  }

}
