<?php

namespace Drupal\o365\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Builds the form to create/edit Office 365 connector entities.
 */
class O365ConnectorForm extends EntityForm {

  /**
   * Constructs an ExampleForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entityTypeManager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);

    /** @var \Drupal\o365\O365ConnectorInterface $o365_connector */
    $o365_connector = $this->entity;

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#default_value' => $o365_connector->label(),
      '#maxlength' => 64,
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $o365_connector->id(),
      '#machine_name' => [
        'exists' => [$this, 'exist'],
      ],
      '#disabled' => !$o365_connector->isNew(),
    ];

    $form['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#description' => $this->t('The client ID as generated on your Microsoft portal.'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $o365_connector->getClientId(),
      '#required' => TRUE,
    ];

    $form['client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client secret'),
      '#description' => $this->t('The client secret as generated on your Microsoft portal. <em>The currently client secret is hidden for security reasons</em>.'),
      '#maxlength' => 255,
      '#size' => 64,
      '#attributes' => [
        'autocomplete' => 'off',
      ],
      '#required' => $this->entity->isNew(),
    ];

    $form['tenant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Tenant identifier'),
      '#description' => $this->t('The directory tenant that you want to login in. This section can be a domain name or tenant object ID. If you want let users sign in with any tenant, use: common.'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $o365_connector->getTenantId(),
      '#required' => TRUE,
    ];

    $form['redirect_login'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Redirect after login URL'),
      '#description' => $this->t('After login users will be redirected to this page. This needs to be a full url like https://www.example.com.'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $o365_connector->getRedirectLogin(),
      '#required' => TRUE,
    ];

    $form['auth_scopes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Authorization scopes'),
      '#description' => $this->t('A space separated list of authorization scopes. The offline_access scope (needed for login purposes) is automatically added.'),
      '#default_value' => $o365_connector->getAuthScopes(),
      '#required' => TRUE,
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $o365_connector->status(),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    if (empty($form_state->getValue('client_secret'))) {
      /** @var \Drupal\o365\O365ConnectorInterface $o365_connector */
      $o365_connector = $this->entityTypeManager->getStorage('o365_connector')
        ->load($this->entity->id());
      $form_state->setValue('client_secret', $o365_connector->getClientSecret());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $status = $this->entity->save();

    if ($status === SAVED_NEW) {
      $this->messenger()
        ->addMessage($this->t('The @label connector created.', [
          '@label' => $this->entity->label(),
        ]));
    }
    else {
      $this->messenger()
        ->addMessage($this->t('The @label connector updated.', [
          '@label' => $this->entity->label(),
        ]));
    }

    $form_state->setRedirect('entity.o365_connector.collection');

    return $status;
  }

  /**
   * Helper function to check whether a Social o365 configuration entity exists.
   *
   * @param string $id
   *   The Office 365 connector ID.
   *
   * @return bool
   *   TRUE if connector exists, FALSE otherwise.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function exist(string $id): bool {
    $entity = $this->entityTypeManager->getStorage('o365_connector')->getQuery()
      ->condition('id', $id)
      ->execute();

    return (bool) $entity;
  }

}
